﻿using System.Collections.Generic;
using System.Web.Mvc;
using Kreta.BusinessLogic.Helpers;
using Kreta.BusinessLogic.Security;
using Kreta.Enums.ManualEnums;
using Kreta.Framework;
using Kreta.Web.Areas.Intezmeny.ApiControllers;
using Kreta.Web.Areas.Intezmeny.Models;
using Kreta.Web.Helpers;
using Kreta.Web.Helpers.TabStrip;
using Kreta.Web.Models.EditorTemplates;
using Kreta.Web.Security;

namespace Kreta.Web.Areas.Intezmeny.Controllers
{
    [MvcRoleClaimsAuthorize(true)]
    [MvcRolePackageDenyAuthorize(KretaClaimPackages.IsOnlyAlkalmozott.ClaimValue)]
    [MvcRolePackageAuthorize(TanevEnum.AktEsKovTanev, KretaClaimPackages.Adminisztrator.ClaimValue)]
    public class CsengetesiRendController : Controller
    {
        // GET: Intezmeny/CsengetesiRend
        public ActionResult Index()
        {
            CsengetesiRendGridModel model = new CsengetesiRendGridModel();

            return View("Index", model);
        }

        public ActionResult OpenCsengRendModositUjPopup(int? csengRendID)
        {
            System.Web.HttpContext.Current.Application["aktualisLista"] = null;

            PopUpModel pm;
            CsengetesiRendModel csmodel;
            CsengetesiRendApiController api = new CsengetesiRendApiController();

            if (csengRendID.HasValue == false)
            {
                csmodel = new CsengetesiRendModel() { ID = 0 };
            }
            else
            {
                csmodel = api.GetCsengetesiRendProperties(csengRendID.Value);
            }

            csmodel.MukodesiHelyList = GetMukodesiHelyList();

            pm = new PopUpModel(csmodel, "CsengetesiRend_Uj_Modositas");

            pm = pm.AddCancelBtn(pm, "CsengRendHelper.modifyAddCancel");
            pm = pm.AddOkBtn(pm, "CsengRendHelper.modifyAddSave");
            return PartialView(Constants.General.PopupView, pm);
        }

        public ActionResult OpenCsengRendTorlesPopup(int csengRendID)
        {
            System.Web.HttpContext.Current.Application["aktualisLista"] = null;

            CsengetesiRendTorlesModel csmodel = new CsengetesiRendTorlesModel()
            {
                ID = csengRendID
            };

            PopUpModel pm = new PopUpModel(csmodel, "CsengetesiRend_Torles");
            pm = pm.AddCancelBtn(pm, "CsengRendHelper.csengRendTorlesCancel");
            pm = pm.AddOkBtn(pm, "CsengRendHelper.validationAndSaveCsengRend");

            return PartialView(Constants.General.PopupView, pm);
        }

        public ActionResult OpenCsengetesiRendAdatokTab(int csrendID)
        {
            System.Web.HttpContext.Current.Application["aktualisLista"] = null;

            CsengetesiRendModel csmodel = new CsengetesiRendModel()
            {
                ID = csrendID
            };
            csmodel.TabList = GetTabItems(csmodel);
            PopUpModel pm = new PopUpModel(csmodel, "CsengetesiRendAdatokTab");
            pm = pm.AddBtn(pm, "BtnCancel", Resources.CommonResource.Vissza, "CsengRendHelper.adatokCancel");

            return PartialView(Constants.General.PopupView, pm);
        }

        public ActionResult OpenCsengetesiRendAlapadatok(int id)
        {
            CsengetesiRendApiController api = new CsengetesiRendApiController();
            CsengetesiRendModel csmodel = api.GetCsengetesiRendProperties(id);

            return PartialView("CsengetesiRendAlapadat_Tab", csmodel);
        }

        public ActionResult OpenCsengetesiRendTanevrendjei(int id)
        {
            CsengetesiRendModel csmodel = new CsengetesiRendModel()
            {
                ID = id
            };
            return PartialView("CsengetesiRendTanevrend_Tab", csmodel);
        }

        public List<TabStripItemModel> GetTabItems(CsengetesiRendModel model)
        {
            var items = new List<TabStripItemModel>();
            items.Add(new TabStripItemModel { ItemId = "1", ItemName = StringResourcesUtil.GetString(161), Area = "Intezmeny", Controller = "CsengetesiRend", Action = "OpenCsengetesiRendAlapadatok", RouteParameters = new Dictionary<string, string>() { { "Id", model.ID.ToString() } } });
            items.Add(new TabStripItemModel { ItemId = "2", ItemName = StringResourcesUtil.GetString(3833), Area = "Intezmeny", Controller = "CsengetesiRend", Action = "OpenCsengetesiRendTanevrendjei", RouteParameters = new Dictionary<string, string>() { { "Id", model.ID.ToString() } } });
            return items;
        }

        private List<SelectListItem> GetMukodesiHelyList()
        {
            MukodesiHelyHelper helper = new MukodesiHelyHelper(ConnectionTypeExtensions.GetActiveSessionConnectionType());

            var dictionary = helper.GetMukodesiHelyekDDL("");
            List<SelectListItem> list = new List<SelectListItem>();

            foreach (var item in dictionary)
            {
                SelectListItem sli = new SelectListItem() { Text = item.Value, Value = item.Key };
                list.Add(sli);
            }
            return list;
        }
    }
}
