﻿using System.Collections.Generic;
using System.Web.Mvc;
using Kreta.BusinessLogic.Helpers;
using Kreta.BusinessLogic.Security;
using Kreta.Core.FeatureToggle;
using Kreta.Core.FileService;
using Kreta.Enums.ManualEnums;
using Kreta.Web.Areas.Intezmeny.Models;
using Kreta.Web.Helpers;
using Kreta.Web.Security;

namespace Kreta.Web.Areas.Intezmeny.Controllers
{
    [MvcRoleClaimsAuthorize(true)]
    [MvcRolePackageDenyAuthorize(KretaClaimPackages.IsOnlyAlkalmozott.ClaimValue, KretaClaimPackages.IsDefaultAdminUser.ClaimValue)]
    [MvcRolePackageAuthorize(TanevEnum.Mind, KretaClaimPackages.Adminisztrator.ClaimValue, KretaClaimPackages.Dualis_Admin.ClaimValue)]
    [Attributes.KretaGlobalLanguageChangeActionFilter(LanguageCode = "hu-Dualis")]
    public class DualisAdminTeremController : BaseTeremController
    {
        public bool IsDualisKepzesEnabled => new IntezmenyConfigHelper(ConnectionTypeExtensions.GetSystemConnectionType()).GetIntezmenyConfig<bool>(IntezmenyConfigModulEnum.DualisKepzes, IntezmenyConfigTipusEnum.IsEnabled);

        public DualisAdminTeremController(IFeatureContext featureContext, IFileService fileService)
            : base(featureContext, fileService)
        {
        }

        public ActionResult OpenTeremPropertiesTab(int teremId)
        {
            return OpenTeremPropertiesTab(teremId, true, Constants.Controllers.DualisAdminTerem);
        }

        public ActionResult OpenTeremBaseProperties(int teremId)
        {
            return OpenTeremBaseProperties(teremId, IsDualisKepzesEnabled);
        }

        public ActionResult OpenTeremOrarendiOrai(int teremId)
        {
            return OpenTeremOrarendiOrai(teremId, IsDualisKepzesEnabled);
        }

        public ActionResult OpenTeremModifyAddTab(int? teremId, int? szervezetId)
        {
            return OpenTeremModifyAddTab(teremId, szervezetId, IsDualisKepzesEnabled);
        }

        public ActionResult OpenModPopup(List<TeremModModel> teremModelList, int? szervezetId)
        {
            return OpenModPopup(teremModelList, szervezetId, IsDualisKepzesEnabled);
        }
    }
}
